@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -22,12 +22,6 @@
  *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef WIN32
-#include "config-win32.h"
-#else
-#include "config.h"
-#endif
-
 #include "syshead.h"
 
 #include "common.h"
@@ -38,6 +32,22 @@
 
 #include "memdbg.h"
 
+size_t
+array_mult_safe (const size_t m1, const size_t m2, const size_t extra)
+{
+  const size_t limit = 0xFFFFFFFF;
+  unsigned long long res = (unsigned long long)m1 * (unsigned long long)m2 + (unsigned long long)extra;
+  if (unlikely(m1 > limit) || unlikely(m2 > limit) || unlikely(extra > limit) || unlikely(res > (unsigned long long)limit))
+    msg (M_FATAL, "attemped allocation of excessively large array");
+  return (size_t) res;
+}
+
+void
+buf_size_error (size_t size)
+{
+  msg (M_FATAL, "fatal buffer size error, size=%lu", (unsigned long)size);
+}
+
 struct buffer
 #ifdef DMALLOC
 alloc_buf_debug (size_t size, const char *file, int line)
@@ -60,6 +70,8 @@ alloc_buf_gc (size_t size, struct gc_arena *gc)
 #endif
 {
   struct buffer buf;
+  if (!buf_size_valid (size))
+    buf_size_error (size);
   buf.capacity = (int)size;
   buf.offset = 0;
   buf.len = 0;
@@ -178,9 +190,10 @@ buf_sub (struct buffer *buf, int size, bool prepend)
 /*
  * printf append to a buffer with overflow check
  */
-void
+bool
 buf_printf (struct buffer *buf, const char *format, ...)
 {
+  int ret = false;
   if (buf_defined (buf))
     {
       va_list arglist;
@@ -189,13 +202,17 @@ buf_printf (struct buffer *buf, const char *format, ...)
 
       if (cap > 0)
 	{
+	  int stat;
 	  va_start (arglist, format);
-	  vsnprintf ((char *)ptr, cap, format, arglist);
+	  stat = vsnprintf ((char *)ptr, cap, format, arglist);
 	  va_end (arglist);
 	  *(buf->data + buf->capacity - 1) = 0; /* windows vsnprintf needs this */
 	  buf->len += (int) strlen ((char *)ptr);
+	  if (stat >= 0 && stat < cap)
+	    ret = true;
 	}
     }
+  return ret;
 }
 
 /*
@@ -320,6 +337,26 @@ x_gc_free (struct gc_arena *a)
     }
 }
 
+/*
+ * Transfer src arena to dest, resetting src to an empty arena.
+ */
+void
+gc_transfer (struct gc_arena *dest, struct gc_arena *src)
+{
+  if (dest && src)
+    {
+      struct gc_entry *e = src->list;
+      if (e)
+	{
+	  while (e->next != NULL)
+	    e = e->next;
+	  e->next = dest->list;
+	  dest->list = src->list;
+	  src->list = NULL;
+	}
+    }
+}
+
 /*
  * Hex dump -- Output a binary buffer to a hex string and return it.
  */
@@ -398,6 +435,19 @@ buf_chomp (struct buffer *buf)
   buf_null_terminate (buf);
 }
 
+const char *
+skip_leading_whitespace (const char *str)
+{
+  while (*str)
+    {
+      const char c = *str;
+      if (!(c == ' ' || c == '\t'))
+	break;
+      ++str;
+    }
+  return str;
+}
+
 /*
  * like buf_null_terminate, but operate on strings
  */
@@ -416,6 +466,15 @@ string_null_terminate (char *str, int len, int capacity)
  */
 void
 chomp (char *str)
+{
+  rm_trailing_chars (str, "\r\n");
+}
+
+/*
+ * Remove trailing chars
+ */
+void
+rm_trailing_chars (char *str, const char *what_to_delete)
 {
   bool modified;
   do {
@@ -424,7 +483,7 @@ chomp (char *str)
     if (len > 0)
       {
 	char *cp = str + (len - 1);
-	if (*cp == '\n' || *cp == '\r')
+	if (strchr (what_to_delete, *cp) != NULL)
 	  {
 	    *cp = '\0';
 	    modified = true;
@@ -460,6 +519,56 @@ string_alloc (const char *str, struct gc_arena *gc)
     return NULL;
 }
 
+/*
+ * Erase all characters in a string
+ */
+void
+string_clear (char *str)
+{
+  if (str)
+    {
+      const int len = strlen (str);
+      if (len > 0)
+	memset (str, 0, len);
+    }
+}
+
+/*
+ * Return the length of a string array
+ */
+int
+string_array_len (const char **array)
+{
+  int i = 0;
+  if (array)
+    {
+      while (array[i])
+        ++i;
+    }
+  return i;
+}
+
+char *
+print_argv (const char **p, struct gc_arena *gc, const unsigned int flags)
+{
+  struct buffer out = alloc_buf_gc (256, gc);
+  int i = 0;
+  for (;;)
+    {
+      const char *cp = *p++;
+      if (!cp)
+	break;
+      if (i)
+	buf_printf (&out, " ");
+      if (flags & PA_BRACKET)
+	buf_printf (&out, "[%s]", cp);
+      else
+	buf_printf (&out, "%s", cp);
+      ++i;
+    }
+  return BSTR (&out);
+}
+
 /*
  * Allocate a string inside a buffer
  */
@@ -557,12 +666,24 @@ buf_parse (struct buffer *buf, const int delim, char *line, const int size)
   return !(eol && !strlen (line));
 }
 
+/*
+ * Print a string which might be NULL
+ */
+const char *
+np (const char *str)
+{
+  if (str)
+    return str;
+  else
+    return "[NULL]";
+}
+
 /*
  * Classify and mutate strings based on character types.
  */
 
 bool
-char_class (const char c, const unsigned int flags)
+char_class (const unsigned char c, const unsigned int flags)
 {
   if (!flags)
     return false;
@@ -696,6 +817,20 @@ string_mod_const (const char *str,
     return NULL;
 }
 
+void
+string_replace_leading (char *str, const char match, const char replace)
+{
+  ASSERT (match != '\0');
+  while (*str)
+    {
+      if (*str == match)
+	*str = replace;
+      else
+	break;
+      ++str;
+    }
+}
+
 #ifdef CHARACTER_CLASS_DEBUG
 
 #define CC_INCLUDE    (CC_PRINT)
@@ -741,3 +876,130 @@ valign4 (const struct buffer *buf, const char *file, const int line)
     }
 }
 #endif
+
+/*
+ * struct buffer_list
+ */
+
+#ifdef ENABLE_BUFFER_LIST
+
+struct buffer_list *
+buffer_list_new (const int max_size)
+{
+  struct buffer_list *ret;
+  ALLOC_OBJ_CLEAR (ret, struct buffer_list);
+  ret->max_size = max_size;
+  ret->size = 0;
+  return ret;
+}
+
+void
+buffer_list_free (struct buffer_list *ol)
+{
+  buffer_list_reset (ol);
+  free (ol);
+}
+
+bool
+buffer_list_defined (const struct buffer_list *ol)
+{
+  return ol->head != NULL;
+}
+
+void
+buffer_list_reset (struct buffer_list *ol)
+{
+  struct buffer_entry *e = ol->head;
+  while (e)
+    {
+      struct buffer_entry *next = e->next;
+      free_buf (&e->buf);
+      free (e);
+      e = next;
+    }
+  ol->head = ol->tail = NULL;
+  ol->size = 0;
+}
+
+void
+buffer_list_push (struct buffer_list *ol, const unsigned char *str)
+{
+  if (!ol->max_size || ol->size < ol->max_size)
+    {
+      struct buffer_entry *e;
+      ALLOC_OBJ_CLEAR (e, struct buffer_entry);
+
+      ++ol->size;
+      if (ol->tail)
+	{
+	  ASSERT (ol->head);
+	  ol->tail->next = e;
+	}
+      else
+	{
+	  ASSERT (!ol->head);
+	  ol->head = e;
+	}
+      e->buf = string_alloc_buf ((const char *) str, NULL);
+      ol->tail = e;
+    }
+}
+
+const struct buffer *
+buffer_list_peek (struct buffer_list *ol)
+{
+  if (ol->head)
+    return &ol->head->buf;
+  else
+    return NULL;
+}
+
+static void
+buffer_list_pop (struct buffer_list *ol)
+{
+  if (ol->head)
+    {
+      struct buffer_entry *e = ol->head->next;
+      free_buf (&ol->head->buf);
+      free (ol->head);
+      ol->head = e;
+      --ol->size;
+      if (!e)
+	ol->tail = NULL;
+    }
+}
+
+void
+buffer_list_advance (struct buffer_list *ol, int n)
+{
+  if (ol->head)
+    {
+      struct buffer *buf = &ol->head->buf;
+      ASSERT (buf_advance (buf, n));
+      if (!BLEN (buf))
+	buffer_list_pop (ol);
+    }
+}
+
+struct buffer_list *
+buffer_list_file (const char *fn, int max_line_len)
+{
+  FILE *fp = fopen (fn, "r");
+  struct buffer_list *bl = NULL;
+
+  if (fp)
+    {
+      char *line = (char *) malloc (max_line_len);
+      if (line)
+	{
+	  bl = buffer_list_new (0);
+	  while (fgets (line, max_line_len, fp) != NULL)
+	    buffer_list_push (bl, (unsigned char *)line);
+	  free (line);
+	}
+      fclose (fp);
+    }
+  return bl;
+}
+
+#endif
