@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -22,16 +22,10 @@
  *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef WIN32
-#include "config-win32.h"
-#else
-#include "config.h"
-#endif
+#include "syshead.h"
 
 #ifdef USE_CRYPTO
 
-#include "syshead.h"
-
 #include "crypto.h"
 #include "error.h"
 #include "misc.h"
@@ -905,11 +899,13 @@ test_crypto (const struct crypto_options *co, struct frame* frame)
 }
 
 #ifdef USE_SSL
+
 void
 get_tls_handshake_key (const struct key_type *key_type,
 		       struct key_ctx_bi *ctx,
 		       const char *passphrase_file,
-		       bool key_direction)
+		       const int key_direction,
+		       const unsigned int flags)
 {
   if (passphrase_file && key_type->hmac_length)
     {
@@ -921,37 +917,52 @@ get_tls_handshake_key (const struct key_type *key_type,
       kt.cipher_length = 0;
       kt.cipher = NULL;
 
-      /* first try to parse as an OpenVPN static key file */
-      read_key_file (&key2, passphrase_file, false);
-
-      /* succeeded? */
-      if (key2.n == 2)
+#if ENABLE_INLINE_FILES
+      if (flags & GHK_INLINE)
 	{
-	  msg (M_INFO,
-	       "Control Channel Authentication: using '%s' as a " PACKAGE_NAME " static key file",
-	       passphrase_file);
+	  /* key was specified inline, key text is in passphrase_file */
+	  read_key_file (&key2, passphrase_file, RKF_INLINE|RKF_MUST_SUCCEED);
+
+	  /* succeeded? */
+	  if (key2.n == 2)
+	    msg (M_INFO, "Control Channel Authentication: tls-auth using INLINE static key file");
+	  else
+	    msg (M_FATAL, "INLINE tls-auth file lacks the requisite 2 keys");
 	}
       else
-	{
-	  int hash_size;
+#endif
+      {
+	/* first try to parse as an OpenVPN static key file */
+	read_key_file (&key2, passphrase_file, 0);
 
-	  CLEAR (key2);
+	/* succeeded? */
+	if (key2.n == 2)
+	  {
+	    msg (M_INFO,
+		 "Control Channel Authentication: using '%s' as a " PACKAGE_NAME " static key file",
+		 passphrase_file);
+	  }
+	else
+	  {
+	    int hash_size;
 
-	  /* failed, now try to get hash from a freeform file */
-	  hash_size = read_passphrase_hash (passphrase_file,
-					    kt.digest,
-					    key2.keys[0].hmac,
-					    MAX_HMAC_KEY_LENGTH);
-	  ASSERT (hash_size == kt.hmac_length);
+	    CLEAR (key2);
 
-	  /* suceeded */
-	  key2.n = 1;
+	    /* failed, now try to get hash from a freeform file */
+	    hash_size = read_passphrase_hash (passphrase_file,
+					      kt.digest,
+					      key2.keys[0].hmac,
+					      MAX_HMAC_KEY_LENGTH);
+	    ASSERT (hash_size == kt.hmac_length);
 
-	  msg (M_INFO,
-	       "Control Channel Authentication: using '%s' as a free-form passphrase file",
-	       passphrase_file);
-	}
+	    /* suceeded */
+	    key2.n = 1;
 
+	    msg (M_INFO,
+		 "Control Channel Authentication: using '%s' as a free-form passphrase file",
+		 passphrase_file);
+	  }
+      }
       /* handle key direction */
 
       key_direction_state_init (&kds, key_direction);
@@ -984,15 +995,18 @@ static const char unprintable_char_fmt[] =
   "Non-Hex, unprintable character (0x%02x) found at line %d in key file '%s' (%d/%d/%d bytes found/min/max)";
 
 /* read key from file */
+
 void
-read_key_file (struct key2 *key2, const char *filename, bool must_succeed)
+read_key_file (struct key2 *key2, const char *file, const unsigned int flags)
 {
   struct gc_arena gc = gc_new ();
-  struct buffer in = alloc_buf_gc (64, &gc);
+  struct buffer in;
   int fd, size;
   uint8_t hex_byte[3] = {0, 0, 0};
+  const char *error_filename = file;
 
   /* parse info */
+  const unsigned char *cp;
   int hb_index = 0;
   int line_num = 1;
   int line_index = 0;
@@ -1019,95 +1033,112 @@ read_key_file (struct key2 *key2, const char *filename, bool must_succeed)
 
   CLEAR (*key2);
 
-  fd = open (filename, O_RDONLY);
-  if (fd == -1)
-    msg (M_ERR, "Cannot open file key file '%s'", filename);
-
-  while ((size = read (fd, in.data, in.capacity)))
+  /*
+   * Key can be provided as a filename in 'file' or if RKF_INLINE
+   * is set, the actual key data itself in ascii form.
+   */
+#if ENABLE_INLINE_FILES
+  if (flags & RKF_INLINE) /* 'file' is a string containing ascii representation of key */
     {
-      const char *cp = (char *)in.data;
-      while (size)
-	{
-	  const char c = *cp;
-
-#if 0
-	  msg (M_INFO, "char='%c' s=%d ln=%d li=%d m=%d c=%d",
-	       c, state, line_num, line_index, match, count);
+      size = strlen (file) + 1;
+      buf_set_read (&in, (const uint8_t *)file, size);
+      error_filename = INLINE_FILE_TAG;
+    }
+  else /* 'file' is a filename which refers to a file containing the ascii key */
 #endif
-
-	  if (c == '\n')
-	    {
-	      line_index = match = 0;
-	      ++line_num;	      
-	    }
-	  else
-	    {
-	      /* first char of new line */
-	      if (!line_index)
-		{
-		  /* first char of line after header line? */
-		  if (state == PARSE_HEAD)
-		    state = PARSE_DATA;
-
-		  /* first char of footer */
-		  if ((state == PARSE_DATA || state == PARSE_DATA_COMPLETE) && c == '-')
-		    state = PARSE_FOOT;
-		}
-
-	      /* compare read chars with header line */
-	      if (state == PARSE_INITIAL)
-		{
-		  if (line_index < hlen && c == static_key_head[line_index])
-		    {
-		      if (++match == hlen)
-			state = PARSE_HEAD;
-		    }
-		}
-
-	      /* compare read chars with footer line */
-	      if (state == PARSE_FOOT)
-		{
-		  if (line_index < flen && c == static_key_foot[line_index])
-		    {
-		      if (++match == flen)
-			state = PARSE_FINISHED;
-		    }
-		}
-
-	      /* reading key */
-	      if (state == PARSE_DATA)
-		{
-		  if (isxdigit(c))
-		    {
-		      ASSERT (hb_index >= 0 && hb_index < 2);
-		      hex_byte[hb_index++] = c;
-		      if (hb_index == 2)
-			{
-			  unsigned int u;
-			  ASSERT(sscanf((const char *)hex_byte, "%x", &u) == 1);
-			  *out++ = u;
-			  hb_index = 0;
-			  if (++count == keylen)
-			    state = PARSE_DATA_COMPLETE;
-			}
-		    }
-		  else if (isspace(c))
-		    ;
-		  else
-		    {
-		      msg (M_FATAL,
-			   (isprint (c) ? printable_char_fmt : unprintable_char_fmt),
-			   c, line_num, filename, count, onekeylen, keylen);
-		    }
-		}
-	      ++line_index;
-	    }
-	  ++cp;
-	  --size;
-	}
+    {
+      in = alloc_buf_gc (2048, &gc);
+      fd = open (file, O_RDONLY);
+      if (fd == -1)
+	msg (M_ERR, "Cannot open file key file '%s'", file);
+      size = read (fd, in.data, in.capacity);
+      if (size < 0)
+	msg (M_FATAL, "Read error on key file ('%s')", file);
+      if (size == in.capacity)
+	msg (M_FATAL, "Key file ('%s') can be a maximum of %d bytes", file, (int)in.capacity);
+      close (fd);
     }
 
-  close (fd);
+  cp = (unsigned char *)in.data;
+  while (size > 0)
+    {
+      const unsigned char c = *cp;
+
+#if 0
+      msg (M_INFO, "char='%c' s=%d ln=%d li=%d m=%d c=%d",
+	   c, state, line_num, line_index, match, count);
+#endif
+
+      if (c == '\n')
+	{
+	  line_index = match = 0;
+	  ++line_num;	      
+	}
+      else
+	{
+	  /* first char of new line */
+	  if (!line_index)
+	    {
+	      /* first char of line after header line? */
+	      if (state == PARSE_HEAD)
+		state = PARSE_DATA;
+
+	      /* first char of footer */
+	      if ((state == PARSE_DATA || state == PARSE_DATA_COMPLETE) && c == '-')
+		state = PARSE_FOOT;
+	    }
+
+	  /* compare read chars with header line */
+	  if (state == PARSE_INITIAL)
+	    {
+	      if (line_index < hlen && c == static_key_head[line_index])
+		{
+		  if (++match == hlen)
+		    state = PARSE_HEAD;
+		}
+	    }
+
+	  /* compare read chars with footer line */
+	  if (state == PARSE_FOOT)
+	    {
+	      if (line_index < flen && c == static_key_foot[line_index])
+		{
+		  if (++match == flen)
+		    state = PARSE_FINISHED;
+		}
+	    }
+
+	  /* reading key */
+	  if (state == PARSE_DATA)
+	    {
+	      if (isxdigit(c))
+		{
+		  ASSERT (hb_index >= 0 && hb_index < 2);
+		  hex_byte[hb_index++] = c;
+		  if (hb_index == 2)
+		    {
+		      unsigned int u;
+		      ASSERT(sscanf((const char *)hex_byte, "%x", &u) == 1);
+		      *out++ = u;
+		      hb_index = 0;
+		      if (++count == keylen)
+			state = PARSE_DATA_COMPLETE;
+		    }
+		}
+	      else if (isspace(c))
+		;
+	      else
+		{
+		  msg (M_FATAL,
+		       (isprint (c) ? printable_char_fmt : unprintable_char_fmt),
+		       c, line_num, error_filename, count, onekeylen, keylen);
+		}
+	    }
+	  ++line_index;
+	}
+      ++cp;
+      --size;
+    }
 
   /*
    * Normally we will read either 1 or 2 keys from file.
@@ -1116,22 +1147,25 @@ read_key_file (struct key2 *key2, const char *filename, bool must_succeed)
 
   ASSERT (key2->n >= 0 && key2->n <= (int) SIZE (key2->keys));
 
-  if (must_succeed)
+  if (flags & RKF_MUST_SUCCEED)
     {
       if (!key2->n)
 	msg (M_FATAL, "Insufficient key material or header text not found found in file '%s' (%d/%d/%d bytes found/min/max)",
-	     filename, count, onekeylen, keylen);
+	     error_filename, count, onekeylen, keylen);
 
       if (state != PARSE_FINISHED)
 	msg (M_FATAL, "Footer text not found in file '%s' (%d/%d/%d bytes found/min/max)",
-	     filename, count, onekeylen, keylen);
+	     error_filename, count, onekeylen, keylen);
     }
 
-  /* zero file read buffer */
-  buf_clear (&in);
+  /* zero file read buffer if not an inline file */
+#if ENABLE_INLINE_FILES
+  if (!(flags & RKF_INLINE))
+#endif
+    buf_clear (&in);
 
   if (key2->n)
-    warn_if_group_others_accessible (filename);
+    warn_if_group_others_accessible (error_filename);
 
 #if 0
   /* DEBUGGING */
@@ -1609,6 +1643,7 @@ void uninit_crypto_lib ()
       engine_initialized = false;
     }
 #endif
+  prng_uninit ();
 }
 
 /*
@@ -1618,33 +1653,73 @@ void uninit_crypto_lib ()
  * IV values and a number of other miscellaneous tasks.
  */
 
-#define NONCE_SECRET_LEN 16
-
-static uint8_t nonce_data [SHA_DIGEST_LENGTH + NONCE_SECRET_LEN]; /* GLOBAL */
+static uint8_t *nonce_data; /* GLOBAL */
+static const EVP_MD *nonce_md = NULL; /* GLOBAL */
+static int nonce_secret_len; /* GLOBAL */
 
 void
-prng_init (void)
+prng_init (const char *md_name, const int nonce_secret_len_parm)
 {
-  if (!RAND_bytes (nonce_data, sizeof(nonce_data)))
-    msg (M_FATAL, "ERROR: Random number generator cannot obtain entropy for PRNG");
+  prng_uninit ();
+  nonce_md = md_name ? get_md (md_name) : NULL;
+  if (nonce_md)
+    {
+      ASSERT (nonce_secret_len_parm >= NONCE_SECRET_LEN_MIN && nonce_secret_len_parm <= NONCE_SECRET_LEN_MAX);
+      nonce_secret_len = nonce_secret_len_parm;
+      {
+	const int size = EVP_MD_size (nonce_md) + nonce_secret_len;
+	dmsg (D_CRYPTO_DEBUG, "PRNG init md=%s size=%d", EVP_MD_name (nonce_md), size);
+	nonce_data = (uint8_t*) malloc (size);
+	check_malloc_return (nonce_data);
+#if 1 /* Must be 1 for real usage */
+	if (!RAND_bytes (nonce_data, size))
+	  msg (M_FATAL, "ERROR: Random number generator cannot obtain entropy for PRNG");
+#else
+	/* Only for testing -- will cause a predictable PRNG sequence */
+	{
+	  int i;
+	  for (i = 0; i < size; ++i)
+	    nonce_data[i] = (uint8_t) i;
+	}
+#endif
+      }
+    }
+}
+
+void
+prng_uninit (void)
+{
+  free (nonce_data);
+  nonce_data = NULL;
+  nonce_md = NULL;
+  nonce_secret_len = 0;
 }
 
 void
 prng_bytes (uint8_t *output, int len)
 {
-  SHA_CTX ctx;
-  mutex_lock_static (L_PRNG);
-  while (len > 0)
+  if (nonce_md)
     {
-      const int blen = min_int (len, SHA_DIGEST_LENGTH);
-      SHA1_Init (&ctx);
-      SHA1_Update (&ctx, nonce_data, sizeof (nonce_data));
-      SHA1_Final (nonce_data, &ctx);
-      memcpy (output, nonce_data, blen);
-      output += blen;
-      len -= blen;
+      EVP_MD_CTX ctx;
+      const int md_size = EVP_MD_size (nonce_md);
+      mutex_lock_static (L_PRNG);
+      while (len > 0)
+	{
+	  unsigned int outlen = 0;
+	  const int blen = min_int (len, md_size);
+	  EVP_DigestInit (&ctx, nonce_md);
+	  EVP_DigestUpdate (&ctx, nonce_data, md_size + nonce_secret_len);
+	  EVP_DigestFinal (&ctx, nonce_data, &outlen);
+	  ASSERT (outlen == md_size);
+	  EVP_MD_CTX_cleanup (&ctx);
+	  memcpy (output, nonce_data, blen);
+	  output += blen;
+	  len -= blen;
+	}
+      mutex_unlock_static (L_PRNG);
     }
-  mutex_unlock_static (L_PRNG);
+  else
+    RAND_bytes (output, len);
 }
 
 /* an analogue to the random() function, but use prng_bytes */
@@ -1721,4 +1796,49 @@ free_ssl_lib (void)
 }
 
 #endif /* USE_SSL */
+
+/*
+ * md5 functions
+ */
+
+void
+md5_state_init (struct md5_state *s)
+{
+  MD5_Init (&s->ctx);
+}
+
+void
+md5_state_update (struct md5_state *s, void *data, size_t len)
+{
+  MD5_Update (&s->ctx, data, len);
+}
+
+void
+md5_state_final (struct md5_state *s, struct md5_digest *out)
+{
+  MD5_Final (out->digest, &s->ctx);
+}
+
+void
+md5_digest_clear (struct md5_digest *digest)
+{
+  CLEAR (*digest);
+}
+
+bool
+md5_digest_defined (const struct md5_digest *digest)
+{
+  int i;
+  for (i = 0; i < MD5_DIGEST_LENGTH; ++i)
+    if (digest->digest[i])
+      return true;
+  return false;
+}
+
+bool
+md5_digest_equal (const struct md5_digest *d1, const struct md5_digest *d2)
+{
+  return memcmp(d1->digest, d2->digest, MD5_DIGEST_LENGTH) == 0;
+}
+
 #endif /* USE_CRYPTO */
