@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -30,7 +30,11 @@
 
 /* #define ABORT_ON_ERROR */
 
-#define ERR_BUF_SIZE 1024
+#ifdef ENABLE_PKCS11
+#define ERR_BUF_SIZE 8192
+#else
+#define ERR_BUF_SIZE 1280
+#endif
 
 struct gc_arena;
 
@@ -38,7 +42,8 @@ struct gc_arena;
  * Where should messages be printed before syslog is opened?
  * Not used if OPENVPN_DEBUG_COMMAND_LINE is defined.
  */
-#define OPENVPN_MSG_FP stdout
+#define OPENVPN_MSG_FP   stdout
+#define OPENVPN_ERROR_FP stderr
 
 /*
  * Exit status codes
@@ -97,13 +102,15 @@ extern int x_msg_line_num;
 #define M_USAGE_SMALL     (1<<13)        /* fatal options error, call usage_small */
 #define M_MSG_VIRT_OUT    (1<<14)        /* output message through msg_status_output callback */
 #define M_OPTERR          (1<<15)        /* print "Options error:" prefix */
+#define M_NOLF            (1<<16)        /* don't print new line */
+#define M_NOIPREFIX       (1<<17)        /* don't print instance prefix */
 
 /* flag combinations which are frequently used */
 #define M_ERR     (M_FATAL | M_ERRNO)
 #define M_SOCKERR (M_FATAL | M_ERRNO_SOCK)
 #define M_SSLERR  (M_FATAL | M_SSL)
 #define M_USAGE   (M_USAGE_SMALL | M_NOPREFIX | M_OPTERR)
-#define M_CLIENT  (M_MSG_VIRT_OUT|M_NOMUTE)
+#define M_CLIENT  (M_MSG_VIRT_OUT | M_NOMUTE | M_NOIPREFIX)
 
 /*
  * Mute levels are designed to avoid large numbers of
@@ -111,7 +118,7 @@ extern int x_msg_line_num;
  *
  * A mute level of 0 is always printed.
  */
-#define MUTE_LEVEL_SHIFT 16
+#define MUTE_LEVEL_SHIFT 24
 #define MUTE_LEVEL_MASK 0xFF
 
 #define ENCODE_MUTE_LEVEL(mute_level) (((mute_level) & MUTE_LEVEL_MASK) << MUTE_LEVEL_SHIFT)
@@ -121,6 +128,11 @@ extern int x_msg_line_num;
  * log_level:  verbosity level n (--verb n) must be >= log_level to print.
  * mute_level: don't print more than n (--mute n) consecutive messages at
  *             a given mute level, or if 0 disable muting and print everything.
+ *
+ * Mask map:
+ * Bits 0-3:   log level
+ * Bits 4-23:  M_x flags
+ * Bits 24-31: mute level
  */
 #define LOGLEV(log_level, mute_level, other) ((log_level) | ENCODE_MUTE_LEVEL(mute_level) | other)
 
@@ -131,7 +143,7 @@ extern int x_msg_line_num;
 
 bool dont_mute (unsigned int flags); /* check muting filter */
 
-#define MSG_TEST(flags) (((((unsigned int)flags) & M_DEBUG_LEVEL) <= x_debug_level) && dont_mute (flags))
+#define MSG_TEST(flags) (unlikely((((unsigned int)flags) & M_DEBUG_LEVEL) <= x_debug_level) && dont_mute (flags))
 
 #if defined(HAVE_CPP_VARARG_MACRO_ISO) && !defined(__LCLINT__)
 # define HAVE_VARARG_MACROS
@@ -172,6 +184,10 @@ void x_msg (const unsigned int flags, const char *format, ...)
  */
 
 void error_reset (void);
+
+/* route errors to stderr that would normally go to stdout */
+void errors_to_stderr (void);
+
 void set_suppress_timestamps (bool suppressed);
 
 #define SDL_CONSTRAIN (1<<0)
@@ -187,13 +203,17 @@ const char *msg_flags_string (const unsigned int flags, struct gc_arena *gc);
 /*
  * File to print messages to before syslog is opened.
  */
-FILE *msg_fp(void);
+FILE *msg_fp(const unsigned int flags);
 
 /* Fatal logic errors */
 #define ASSERT(x) do { if (!(x)) assert_failed(__FILE__, __LINE__); } while (false)
 
 void assert_failed (const char *filename, int line);
 
+#ifdef ENABLE_DEBUG
+void crash (void); /* force a segfault (debugging only) */
+#endif
+
 /* Inline functions */
 
 static inline bool
@@ -202,6 +222,9 @@ check_debug_level (unsigned int level)
   return (level & M_DEBUG_LEVEL) <= x_debug_level;
 }
 
+/* Call if we forked */
+void msg_forked (void);
+
 /* syslog output */
 
 void open_syslog (const char *pgmname, bool stdio_to_null);
