@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -77,7 +77,7 @@ void do_up (struct context *c,
 	    bool pulled_options,
 	    unsigned int option_types_found);
 
-unsigned int pull_permission_mask (void);
+unsigned int pull_permission_mask (const struct context *c);
 
 const char *format_common_name (struct context *c, struct gc_arena *gc);
 
@@ -119,4 +119,9 @@ void management_show_net_callback (void *arg, const int msglevel);
 void init_management_callback_p2p (struct context *c);
 void uninit_management_callback (void);
 
+#ifdef ENABLE_PLUGIN
+void init_plugins (struct context *c);
+void open_plugins (struct context *c, const bool import_options, int init_point);
+#endif
+
 #endif
