@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -77,8 +77,13 @@ struct multi_instance {
   bool did_open_context;
   bool did_real_hash;
   bool did_iter;
+#ifdef MANAGEMENT_DEF_AUTH
+  bool did_cid_hash;
+  struct buffer_list *cc_config;
+#endif
   bool connection_established_flag;
   bool did_iroutes;
+  int n_clients_delta; /* added to multi_context.n_clients when instance is closed */
 
   struct context context;
 };
@@ -110,6 +115,12 @@ struct multi_context {
   int max_clients;
   int tcp_queue_limit;
   int status_file_version;
+  int n_clients; /* current number of authenticated clients */
+
+#ifdef MANAGEMENT_DEF_AUTH
+  struct hash *cid_hash;
+  unsigned long cid_counter;
+#endif
 
   struct multi_instance *pending;
   struct multi_instance *earliest_wakeup;
@@ -143,10 +154,6 @@ void tunnel_server (struct context *top);
 
 const char *multi_instance_string (const struct multi_instance *mi, bool null, struct gc_arena *gc);
 
-void multi_bcast (struct multi_context *m,
-		  const struct buffer *buf,
-		  struct multi_instance *omit);
-
 /*
  * Called by mtcp.c, mudp.c, or other (to be written) protocol drivers
  */
@@ -376,7 +383,7 @@ multi_get_timeout (struct multi_context *m, struct timeval *dest)
   m->earliest_wakeup = (struct multi_instance *) schedule_get_earliest_wakeup (m->schedule, &tv);
   if (m->earliest_wakeup)
     {
-      ASSERT (!gettimeofday (&current, NULL));
+      ASSERT (!openvpn_gettimeofday (&current, NULL));
       tv_delta (dest, &current, &tv);
       if (dest->tv_sec >= REAP_MAX_WAKEUP)
 	{
