@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -22,12 +22,6 @@
  *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef WIN32
-#include "config-win32.h"
-#else
-#include "config.h"
-#endif
-
 #include "syshead.h"
 
 #ifdef ENABLE_OCC
@@ -155,7 +149,7 @@ check_send_occ_req_dowork (struct context *c)
 {
   if (++c->c2.occ_n_tries >= OCC_N_TRIES)
     {
-      if (c->options.remote_list)
+      if (c->options.ce.remote)
 	/*
 	 * No OCC_REPLY from peer after repeated attempts.
 	 * Give up.
@@ -375,7 +369,7 @@ process_received_occ_msg (struct context *c)
 	       c->c2.max_send_size_remote,
 	       c->c2.max_recv_size_local);
 	  if (!c->options.fragment
-	      && c->options.proto == PROTO_UDPv4
+	      && c->options.ce.proto == PROTO_UDPv4
 	      && c->c2.max_send_size_local > TUN_MTU_MIN
 	      && (c->c2.max_recv_size_remote < c->c2.max_send_size_local
 		  || c->c2.max_recv_size_local < c->c2.max_send_size_remote))
