@@ -5,7 +5,7 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2005 OpenVPN Solutions LLC <info@openvpn.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -41,28 +41,17 @@
  */
 #define OPENVPN_PORT 1194
 
+/*
+ * Maximum size passed passed to setsockopt SNDBUF/RCVBUF
+ */
+#define SOCKET_SND_RCV_BUF_MAX 1000000
+
 /*
  * Number of seconds that "resolv-retry infinite"
  * represents.
  */
 #define RESOLV_RETRY_INFINITE 1000000000
 
-#define REMOTE_LIST_SIZE 64
-
-struct remote_entry
-{
-  const char *hostname;
-  int port;
-};
-
-struct remote_list
-{
-  int len;
-  int current;
-  bool no_advance;
-  struct remote_entry array[REMOTE_LIST_SIZE];
-};
-
 /* 
  * packet_size_type is used to communicate packet size
  * over the wire when stream oriented protocols are
@@ -77,12 +66,29 @@ typedef uint16_t packet_size_type;
 /* convert a packet_size_type from network to host order */
 #define ntohps(x) ntohs(x)
 
+/* OpenVPN sockaddr struct */
+struct openvpn_sockaddr
+{
+  /*int dummy;*/ /* add offset to force a bug if sa not explicitly dereferenced */
+  struct sockaddr_in sa;
+};
+
+/* actual address of remote, based on source address of received packets */
+struct link_socket_actual
+{
+  /*int dummy;*/ /* add offset to force a bug if dest not explicitly dereferenced */
+  struct openvpn_sockaddr dest;
+#if ENABLE_IP_PKTINFO
+  struct in_pktinfo pi;
+#endif
+};
+
 /* IP addresses which are persistant across SIGUSR1s */
 struct link_socket_addr
 {
-  struct sockaddr_in local;
-  struct sockaddr_in remote; /* initial remote */
-  struct sockaddr_in actual; /* remote may change due to --float */
+  struct openvpn_sockaddr local;
+  struct openvpn_sockaddr remote;   /* initial remote */
+  struct link_socket_actual actual; /* reply to this address */
 };
 
 struct link_socket_info
@@ -113,6 +119,12 @@ struct stream_buf
 
   bool error;  /* if true, fatal TCP error has occurred,
 		  requiring that connection be restarted */
+#if PORT_SHARE
+# define PS_DISABLED 0
+# define PS_ENABLED  1
+# define PS_FOREIGN  2
+  int port_share_state;
+#endif
 };
 
 /*
@@ -152,8 +164,9 @@ struct link_socket
   /* used for long-term queueing of pre-accepted socket listen */
   bool listen_persistent_queued;
 
-  /* set on initial call to init phase 1 */
-  struct remote_list *remote_list;
+  /* Does config file contain any <connection> ... </connection> blocks? */
+  bool connection_profiles_defined;
+
   const char *remote_host;
   int remote_port;
   const char *local_host;
@@ -172,6 +185,8 @@ struct link_socket
 
   int resolve_retry_seconds;
   int connect_retry_seconds;
+  int connect_timeout;
+  int connect_retry_max;
   int mtu_discover_type;
 
   struct socket_buffer_size socket_buffer_sizes;
@@ -180,6 +195,12 @@ struct link_socket
 
   bool did_resolve_remote;
 
+# define SF_USE_IP_PKTINFO (1<<0)
+# define SF_TCP_NODELAY (1<<1)
+# define SF_PORT_SHARE (1<<2)
+# define SF_HOST_RANDOMIZE (1<<3)
+  unsigned int sockflags;
+
   /* for stream sockets */
   struct stream_buf stream_buf;
   struct buffer stream_buf_data;
@@ -193,7 +214,7 @@ struct link_socket
 #ifdef ENABLE_SOCKS
   /* Socks proxy */
   struct socks_proxy_info *socks_proxy;
-  struct sockaddr_in socks_relay; /* Socks UDP relay address */
+  struct link_socket_actual socks_relay; /* Socks UDP relay address */
 #endif
 
 #if defined(ENABLE_HTTP_PROXY) || defined(ENABLE_SOCKS)
@@ -229,13 +250,13 @@ int socket_recv_queue (struct link_socket *sock, int maxsize);
 
 int socket_send_queue (struct link_socket *sock,
 		       struct buffer *buf,
-		       const struct sockaddr_in *to);
+		       const struct link_socket_actual *to);
 
 int socket_finalize (
 		     SOCKET s,
 		     struct overlapped_io *io,
 		     struct buffer *buf,
-		     struct sockaddr_in *from);
+		     struct link_socket_actual *from);
 
 #else
 
@@ -245,15 +266,26 @@ int socket_finalize (
 
 struct link_socket *link_socket_new (void);
 
+void socket_bind (socket_descriptor_t sd,
+		  struct openvpn_sockaddr *local,
+		  const char *prefix);
+
+int openvpn_connect (socket_descriptor_t sd,
+		     struct openvpn_sockaddr *remote,
+		     int connect_timeout,
+		     volatile int *signal_received);
+
 /*
  * Initialize link_socket object.
  */
 
 void
 link_socket_init_phase1 (struct link_socket *sock,
+			 const bool connection_profiles_defined,
 			 const char *local_host,
-			 struct remote_list *remote_list,
 			 int local_port,
+			 const char *remote_host,
+			 int remote_port,
 			 int proto,
 			 int mode,
 			 const struct link_socket *accept_from,
@@ -274,31 +306,48 @@ link_socket_init_phase1 (struct link_socket *sock,
 			 const struct plugin_list *plugins,
 			 int resolve_retry_seconds,
 			 int connect_retry_seconds,
+			 int connect_timeout,
+			 int connect_retry_max,
 			 int mtu_discover_type,
 			 int rcvbuf,
-			 int sndbuf);
+			 int sndbuf,
+			 unsigned int sockflags);
 
 void link_socket_init_phase2 (struct link_socket *sock,
 			      const struct frame *frame,
 			      volatile int *signal_received);
 
-void link_socket_post_fork (const struct link_socket *sock,
-			    const struct sockaddr_in *remote);
-
 void socket_adjust_frame_parameters (struct frame *frame, int proto);
 
 void frame_adjust_path_mtu (struct frame *frame, int pmtu, int proto);
 
 void link_socket_close (struct link_socket *sock);
 
-const char *print_sockaddr_ex (const struct sockaddr_in *addr,
-			       bool do_port,
+void sd_close (socket_descriptor_t *sd);
+
+#define PS_SHOW_PORT_IF_DEFINED (1<<0)
+#define PS_SHOW_PORT            (1<<1)
+#define PS_SHOW_PKTINFO         (1<<2)
+#define PS_DONT_SHOW_ADDR       (1<<3)
+
+const char *print_sockaddr_ex (const struct openvpn_sockaddr *addr,
 			       const char* separator,
+			       const unsigned int flags,
 			       struct gc_arena *gc);
 
-const char *print_sockaddr (const struct sockaddr_in *addr,
+
+const char *print_sockaddr (const struct openvpn_sockaddr *addr,
 			    struct gc_arena *gc);
 
+const char *print_link_socket_actual_ex (const struct link_socket_actual *act,
+					 const char* separator,
+					 const unsigned int flags,
+					 struct gc_arena *gc);
+
+const char *print_link_socket_actual (const struct link_socket_actual *act,
+				      struct gc_arena *gc);
+
+
 #define IA_EMPTY_IF_UNDEF (1<<0)
 #define IA_NET_ORDER      (1<<1)
 const char *print_in_addr_t (in_addr_t addr, unsigned int flags, struct gc_arena *gc);
@@ -307,7 +356,7 @@ const char *print_in_addr_t (in_addr_t addr, unsigned int flags, struct gc_arena
 #define SA_SET_IF_NONZERO (1<<1)
 void setenv_sockaddr (struct env_set *es,
 		      const char *name_prefix,
-		      const struct sockaddr_in *addr,
+		      const struct openvpn_sockaddr *addr,
 		      const bool flags);
 
 void setenv_in_addr_t (struct env_set *es,
@@ -315,25 +364,31 @@ void setenv_in_addr_t (struct env_set *es,
 		       in_addr_t addr,
 		       const bool flags);
 
+void setenv_link_socket_actual (struct env_set *es,
+				const char *name_prefix,
+				const struct link_socket_actual *act,
+				const bool flags);
+
 void bad_address_length (int actual, int expected);
 
 in_addr_t link_socket_current_remote (const struct link_socket_info *info);
 
 void link_socket_connection_initiated (const struct buffer *buf,
 				       struct link_socket_info *info,
-				       const struct sockaddr_in *addr,
+				       const struct link_socket_actual *addr,
 				       const char *common_name,
 				       struct env_set *es);
 
 void link_socket_bad_incoming_addr (struct buffer *buf,
 				    const struct link_socket_info *info,
-				    const struct sockaddr_in *from_addr);
+				    const struct link_socket_actual *from_addr);
 
 void link_socket_bad_outgoing_addr (void);
 
 void setenv_trusted (struct env_set *es, const struct link_socket_info *info);
 
-void remote_list_randomize (struct remote_list *l);
+bool link_socket_update_flags (struct link_socket *ls, unsigned int sockflags);
+void link_socket_update_buffer_sizes (struct link_socket *ls, int rcvbuf, int sndbuf);
 
 /*
  * Low-level functions
@@ -345,12 +400,41 @@ void remote_list_randomize (struct remote_list *l);
 #define OIA_ERROR     -1
 int openvpn_inet_aton (const char *dotted_quad, struct in_addr *addr);
 
+/* integrity validation on pulled options */
+bool ip_addr_dotted_quad_safe (const char *dotted_quad);
+bool ip_or_dns_addr_safe (const char *addr, const bool allow_fqdn);
+bool mac_addr_safe (const char *mac_addr);
+
 socket_descriptor_t create_socket_tcp (void);
 
 socket_descriptor_t socket_do_accept (socket_descriptor_t sd,
-				      struct sockaddr_in *remote,
+				      struct link_socket_actual *act,
 				      const bool nowait);
 
+#if UNIX_SOCK_SUPPORT
+
+socket_descriptor_t create_socket_unix (void);
+
+void socket_bind_unix (socket_descriptor_t sd,
+		       struct sockaddr_un *local,
+		       const char *prefix);
+
+socket_descriptor_t socket_accept_unix (socket_descriptor_t sd,
+					struct sockaddr_un *remote);
+
+int socket_connect_unix (socket_descriptor_t sd,
+			 struct sockaddr_un *remote);
+
+void sockaddr_unix_init (struct sockaddr_un *local, const char *path);
+
+const char *sockaddr_unix_name (const struct sockaddr_un *local, const char *null);
+
+void socket_delete_unix (const struct sockaddr_un *local);
+
+bool unix_socket_get_peer_uid_gid (const socket_descriptor_t sd, int *uid, int *gid);
+
+#endif
+
 /*
  * DNS resolution
  */
@@ -363,6 +447,8 @@ socket_descriptor_t socket_do_accept (socket_descriptor_t sd,
 #define GETADDR_WARN_ON_SIGNAL        (1<<5)
 #define GETADDR_MSG_VIRT_OUT          (1<<6)
 #define GETADDR_TRY_ONCE              (1<<7)
+#define GETADDR_UPDATE_MANAGEMENT_STATE (1<<8)
+#define GETADDR_RANDOMIZE             (1<<9)
 
 in_addr_t getaddr (unsigned int flags,
 		   const char *hostname,
@@ -392,11 +478,7 @@ int proto_remote (int proto, bool remote);
 #define IPv4_TCP_HEADER_SIZE              40
 #define IPv6_UDP_HEADER_SIZE              40
 
-static const int proto_overhead[] = { /* indexed by PROTO_x */
-  IPv4_UDP_HEADER_SIZE,
-  IPv4_TCP_HEADER_SIZE,
-  IPv4_TCP_HEADER_SIZE
-};
+extern const int proto_overhead[];
 
 static inline int
 datagram_overhead (int proto)
@@ -409,15 +491,6 @@ datagram_overhead (int proto)
  * Misc inline functions
  */
 
-static inline int
-remote_list_len (const struct remote_list *rl)
-{
-  if (rl)
-    return rl->len;
-  else
-    return 0;
-}
-
 static inline bool
 legal_ipv4_port (int port)
 {
@@ -440,33 +513,39 @@ link_socket_connection_oriented (const struct link_socket *sock)
 }
 
 static inline bool
-addr_defined (const struct sockaddr_in *addr)
+addr_defined (const struct openvpn_sockaddr *addr)
 {
-  return addr->sin_addr.s_addr != 0;
+  return addr->sa.sin_addr.s_addr != 0;
 }
 
 static inline bool
-addr_match (const struct sockaddr_in *a1, const struct sockaddr_in *a2)
+link_socket_actual_defined (const struct link_socket_actual *act)
 {
-  return a1->sin_addr.s_addr == a2->sin_addr.s_addr;
+  return act && addr_defined (&act->dest);
+}
+
+static inline bool
+addr_match (const struct openvpn_sockaddr *a1, const struct openvpn_sockaddr *a2)
+{
+  return a1->sa.sin_addr.s_addr == a2->sa.sin_addr.s_addr;
 }
 
 static inline in_addr_t
-addr_host (const struct sockaddr_in *s)
+addr_host (const struct openvpn_sockaddr *s)
 {
-  return ntohl (s->sin_addr.s_addr);
+  return ntohl (s->sa.sin_addr.s_addr);
 }
 
 static inline bool
-addr_port_match (const struct sockaddr_in *a1, const struct sockaddr_in *a2)
+addr_port_match (const struct openvpn_sockaddr *a1, const struct openvpn_sockaddr *a2)
 {
-  return a1->sin_addr.s_addr == a2->sin_addr.s_addr
-    && a1->sin_port == a2->sin_port;
+  return a1->sa.sin_addr.s_addr == a2->sa.sin_addr.s_addr
+    && a1->sa.sin_port == a2->sa.sin_port;
 }
 
 static inline bool
-addr_match_proto (const struct sockaddr_in *a1,
-		  const struct sockaddr_in *a2,
+addr_match_proto (const struct openvpn_sockaddr *a1,
+		  const struct openvpn_sockaddr *a2,
 		  const int proto)
 {
   return link_socket_proto_connection_oriented (proto)
@@ -474,6 +553,35 @@ addr_match_proto (const struct sockaddr_in *a1,
     : addr_port_match (a1, a2);
 }
 
+static inline bool
+link_socket_actual_match (const struct link_socket_actual *a1, const struct link_socket_actual *a2)
+{
+  return addr_port_match (&a1->dest, &a2->dest);
+}
+
+#if PORT_SHARE
+
+static inline bool
+socket_foreign_protocol_detected (const struct link_socket *sock)
+{
+  return link_socket_connection_oriented (sock)
+    && sock->stream_buf.port_share_state == PS_FOREIGN;
+}
+
+static inline const struct buffer *
+socket_foreign_protocol_head (const struct link_socket *sock)
+{
+  return &sock->stream_buf.buf;
+}
+
+static inline int
+socket_foreign_protocol_sd (const struct link_socket *sock)
+{
+  return sock->sd;
+}
+
+#endif
+
 static inline bool
 socket_connection_reset (const struct link_socket *sock, int status)
 {
@@ -497,17 +605,17 @@ socket_connection_reset (const struct link_socket *sock, int status)
 static inline bool
 link_socket_verify_incoming_addr (struct buffer *buf,
 				  const struct link_socket_info *info,
-				  const struct sockaddr_in *from_addr)
+				  const struct link_socket_actual *from_addr)
 {
   if (buf->len > 0)
     {
-      if (from_addr->sin_family != AF_INET)
+      if (from_addr->dest.sa.sin_family != AF_INET)
 	return false;
-      if (!addr_defined (from_addr))
+      if (!link_socket_actual_defined (from_addr))
 	return false;
       if (info->remote_float || !addr_defined (&info->lsa->remote))
 	return true;
-      if (addr_match_proto (from_addr, &info->lsa->remote, info->proto))
+      if (addr_match_proto (&from_addr->dest, &info->lsa->remote, info->proto))
 	return true;
     }
   return false;
@@ -516,21 +624,18 @@ link_socket_verify_incoming_addr (struct buffer *buf,
 static inline void
 link_socket_get_outgoing_addr (struct buffer *buf,
 			      const struct link_socket_info *info,
-			      struct sockaddr_in *addr)
+			      struct link_socket_actual **act)
 {
   if (buf->len > 0)
     {
       struct link_socket_addr *lsa = info->lsa;
-      if (addr_defined (&lsa->actual))
-	{
-	  addr->sin_family = lsa->actual.sin_family;
-	  addr->sin_addr.s_addr = lsa->actual.sin_addr.s_addr;
-	  addr->sin_port = lsa->actual.sin_port;
-	}
+      if (link_socket_actual_defined (&lsa->actual))
+	*act = &lsa->actual;
       else
 	{
 	  link_socket_bad_outgoing_addr ();
 	  buf->len = 0;
+	  *act = NULL;
 	}
     }
 }
@@ -538,7 +643,7 @@ link_socket_get_outgoing_addr (struct buffer *buf,
 static inline void
 link_socket_set_outgoing_addr (const struct buffer *buf,
 			       struct link_socket_info *info,
-			       const struct sockaddr_in *addr,
+			       const struct link_socket_actual *act,
 			       const char *common_name,
 			       struct env_set *es)
 {
@@ -548,14 +653,14 @@ link_socket_set_outgoing_addr (const struct buffer *buf,
       if (
 	  /* new or changed address? */
 	  (!info->connection_established
-	   || !addr_match_proto (addr, &lsa->actual, info->proto))
+	   || !addr_match_proto (&act->dest, &lsa->actual.dest, info->proto))
 	  /* address undef or address == remote or --float */
 	  && (info->remote_float
 	      || !addr_defined (&lsa->remote)
-	      || addr_match_proto (addr, &lsa->remote, info->proto))
+	      || addr_match_proto (&act->dest, &lsa->remote, info->proto))
 	  )
 	{
-	  link_socket_connection_initiated (buf, info, addr, common_name, es);
+	  link_socket_connection_initiated (buf, info, act, common_name, es);
 	}
     }
 }
@@ -566,7 +671,11 @@ link_socket_set_outgoing_addr (const struct buffer *buf,
  * such as TCP.
  */
 
-void stream_buf_init (struct stream_buf *sb, struct buffer *buf);
+void stream_buf_init (struct stream_buf *sb,
+		      struct buffer *buf,
+		      const unsigned int sockflags,
+		      const int proto);
+
 void stream_buf_close (struct stream_buf* sb);
 bool stream_buf_added (struct stream_buf *sb, int length_added);
 
@@ -592,7 +701,7 @@ int link_socket_read_tcp (struct link_socket *sock,
 static inline int
 link_socket_read_udp_win32 (struct link_socket *sock,
 			    struct buffer *buf,
-			    struct sockaddr_in *from)
+			    struct link_socket_actual *from)
 {
   return socket_finalize (sock->sd, &sock->reads, buf, from);
 }
@@ -602,7 +711,7 @@ link_socket_read_udp_win32 (struct link_socket *sock,
 int link_socket_read_udp_posix (struct link_socket *sock,
 				struct buffer *buf,
 				int maxsize,
-				struct sockaddr_in *from);
+				struct link_socket_actual *from);
 
 #endif
 
@@ -611,7 +720,7 @@ static inline int
 link_socket_read (struct link_socket *sock,
 		  struct buffer *buf,
 		  int maxsize,
-		  struct sockaddr_in *from)
+		  struct link_socket_actual *from)
 {
   if (sock->info.proto == PROTO_UDPv4)
     {
@@ -627,7 +736,7 @@ link_socket_read (struct link_socket *sock,
   else if (sock->info.proto == PROTO_TCPv4_SERVER || sock->info.proto == PROTO_TCPv4_CLIENT)
     {
       /* from address was returned by accept */
-      *from = sock->info.lsa->actual;
+      from->dest.sa = sock->info.lsa->actual.dest.sa;
       return link_socket_read_tcp (sock, buf);
     }
   else
@@ -643,14 +752,14 @@ link_socket_read (struct link_socket *sock,
 
 int link_socket_write_tcp (struct link_socket *sock,
 			   struct buffer *buf,
-			   struct sockaddr_in *to);
+			   struct link_socket_actual *to);
 
 #ifdef WIN32
 
 static inline int
 link_socket_write_win32 (struct link_socket *sock,
 			 struct buffer *buf,
-			 struct sockaddr_in *to)
+			 struct link_socket_actual *to)
 {
   int err = 0;
   int status = 0;
@@ -675,17 +784,26 @@ link_socket_write_win32 (struct link_socket *sock,
 static inline int
 link_socket_write_udp_posix (struct link_socket *sock,
 			     struct buffer *buf,
-			     struct sockaddr_in *to)
+			     struct link_socket_actual *to)
 {
-  return sendto (sock->sd, BPTR (buf), BLEN (buf), 0,
-		 (struct sockaddr *) to,
-		 (socklen_t) sizeof (*to));
+#if ENABLE_IP_PKTINFO
+  int link_socket_write_udp_posix_sendmsg (struct link_socket *sock,
+					   struct buffer *buf,
+					   struct link_socket_actual *to);
+
+  if (sock->sockflags & SF_USE_IP_PKTINFO)
+    return link_socket_write_udp_posix_sendmsg (sock, buf, to);
+  else
+#endif
+    return sendto (sock->sd, BPTR (buf), BLEN (buf), 0,
+		   (struct sockaddr *) &to->dest.sa,
+		   (socklen_t) sizeof (to->dest.sa));
 }
 
 static inline int
 link_socket_write_tcp_posix (struct link_socket *sock,
 			     struct buffer *buf,
-			     struct sockaddr_in *to)
+			     struct link_socket_actual *to)
 {
   return send (sock->sd, BPTR (buf), BLEN (buf), MSG_NOSIGNAL);
 }
@@ -695,7 +813,7 @@ link_socket_write_tcp_posix (struct link_socket *sock,
 static inline int
 link_socket_write_udp (struct link_socket *sock,
 		       struct buffer *buf,
-		       struct sockaddr_in *to)
+		       struct link_socket_actual *to)
 {
 #ifdef WIN32
   return link_socket_write_win32 (sock, buf, to);
@@ -708,7 +826,7 @@ link_socket_write_udp (struct link_socket *sock,
 static inline int
 link_socket_write (struct link_socket *sock,
 		   struct buffer *buf,
-		   struct sockaddr_in *to)
+		   struct link_socket_actual *to)
 {
   if (sock->info.proto == PROTO_UDPv4)
     {
