@@ -5,12 +5,11 @@
  *             packet encryption, packet authentication, and
  *             packet compression.
  *
- *  Copyright (C) 2002-2004 James Yonan <jim@yonan.net>
+ *  Copyright (C) 2002-2009 OpenVPN Technologies, Inc. <sales@openvpn.net>
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2
+ *  as published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -23,12 +22,6 @@
  *  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#ifdef WIN32
-#include "config-win32.h"
-#else
-#include "config.h"
-#endif
-
 #include "syshead.h"
 
 #include "status.h"
@@ -219,7 +212,7 @@ status_close (struct status_output *so)
   return ret;
 }
 
-#define STATUS_PRINTF_MAXLEN 256
+#define STATUS_PRINTF_MAXLEN 512
 
 void
 status_printf (struct status_output *so, const char *format, ...)
@@ -228,28 +221,32 @@ status_printf (struct status_output *so, const char *format, ...)
     {
       char buf[STATUS_PRINTF_MAXLEN+2]; /* leave extra bytes for CR, LF */
       va_list arglist;
+      int stat;
 
       va_start (arglist, format);
-      vsnprintf (buf, STATUS_PRINTF_MAXLEN, format, arglist);
+      stat = vsnprintf (buf, STATUS_PRINTF_MAXLEN, format, arglist);
       va_end (arglist);
       buf[STATUS_PRINTF_MAXLEN - 1] = 0;
 
-      if (so->msglevel >= 0)
+      if (stat < 0 || stat >= STATUS_PRINTF_MAXLEN)
+	so->errors = true;
+
+      if (so->msglevel >= 0 && !so->errors)
 	msg (so->msglevel, "%s", buf);
 
-      if (so->fd >= 0)
+      if (so->fd >= 0 && !so->errors)
 	{
 	  int len;
 	  strcat (buf, "\n");
 	  len = strlen (buf);
 	  if (len > 0)
 	    {
-	      if (write (so->fd, buf, len) < 0)
+	      if (write (so->fd, buf, len) != len)
 		so->errors = true;
 	    }
 	}
 
-      if (so->vout)
+      if (so->vout && !so->errors)
 	{
 	  chomp (buf);
 	  (*so->vout->func) (so->vout->arg, so->vout->flags_default, buf);
