@@ -1,14 +1,10 @@
 /*
- *  TAP-Win32 -- A kernel driver to provide virtual tap device functionality
- *               on Windows.  Originally derived from the CIPE-Win32
- *               project by Damion K. Wilson, with extensive modifications by
- *               James Yonan.
+ *  TAP-Win32/TAP-Win64 -- A kernel driver to provide virtual tap
+ *                         device functionality on Windows.
  *
- *  All source code which derives from the CIPE-Win32 project is
- *  Copyright (C) Damion K. Wilson, 2003, and is released under the
- *  GPL version 2 (see below).
+ *  This code was inspired by the CIPE-Win32 driver by Damion K. Wilson.
  *
- *  All other source code is Copyright (C) 2002-2005 OpenVPN Solutions LLC,
+ *  This source code is Copyright (C) 2002-2009 OpenVPN Technologies, Inc.,
  *  and is released under the GPL version 2 (see below).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -171,13 +167,32 @@ VOID SetMediaStatus
     BOOLEAN state
    );
 
-VOID InjectPacket
+VOID InjectPacketDeferred
    (
     TapAdapterPointer p_Adapter,
     UCHAR *packet,
     const unsigned int len
    );
 
+VOID InjectPacketNow
+   (
+    TapAdapterPointer p_Adapter,
+    UCHAR *packet,
+    const unsigned int len
+   );
+
+// for KDEFERRED_ROUTINE and Static Driver Verifier
+//#include <wdm.h>
+//KDEFERRED_ROUTINE InjectPacketDpc;
+
+VOID InjectPacketDpc
+   (
+    KDPC *Dpc,
+    PVOID DeferredContext,
+    PVOID SystemArgument1,
+    PVOID SystemArgument2
+    );
+
 VOID CheckIfDhcpAndTunMode
    (
     TapAdapterPointer p_Adapter
@@ -187,8 +202,26 @@ VOID HookDispatchFunctions();
 
 #if ENABLE_NONADMIN
 
+#if DDKVER_MAJOR < 5600
+/*
+ * Better solution for use on Vista DDK, but possibly not compatible with
+ * earlier DDKs:
+ *
+ * Eliminate the definition of SECURITY_DESCRIPTOR (and even ZwSetSecurityObject),
+ * and at the top of tapdrv.c change:
+ *
+ * #include <ndis.h>
+ * #include <ntstrsafe.h>
+ * #include <ntddk.h>
+ *
+ * To
+ *
+ * #include <ntifs.h>
+ * #include <ndis.h>
+ * #include <ntstrsafe.h>
+ */
 typedef struct _SECURITY_DESCRIPTOR {
-  unsigned char opaque[20];
+  unsigned char opaque[64];
 } SECURITY_DESCRIPTOR;
 
 NTSYSAPI
@@ -199,10 +232,18 @@ ZwSetSecurityObject (
   IN SECURITY_INFORMATION  SecurityInformation,
   IN PSECURITY_DESCRIPTOR  SecurityDescriptor);
 
+#endif
+
 VOID AllowNonAdmin (TapExtensionPointer p_Extension);
 
 #endif
 
+struct WIN2K_NDIS_MINIPORT_BLOCK
+{
+  unsigned char  opaque[16];
+  UNICODE_STRING MiniportName;       // how mini-port refers to us
+};
+
 #if PACKET_TRUNCATION_CHECK
 
 VOID IPv4PacketSizeVerify
